#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.

""" Classes for notification """

# pylint: disable=C0103, E0401, R0903, W1201

import logging
import smtplib
from typing import Dict

import requests
from pycapo import CapoConfig

from workspaces.notification.services.interfaces import (
    NotificationInfoIF,
    NotificationServiceIF,
)

logger = logging.getLogger(__name__)
NOTIF_SETTINGS_KEY = "edu.nrao.workspaces.NotificationSettings"


class NotificationServiceRESTClient(NotificationServiceIF):
    """REST endpoint for NotificationService"""

    def __init__(self):
        self.url = CapoConfig().settings(NOTIF_SETTINGS_KEY).serviceUrl
        self.linkUrl = CapoConfig().settings("edu.nrao.workspaces.UISettings").serviceUrl
        self.sendPI = CapoConfig().getboolean("edu.nrao.workspaces.NotificationSettings.sendPIEmail")

    def send_email(self, notification: str, arguments: Dict):
        requests.post(f"{self.url}/notify/{notification}/send", json=arguments).raise_for_status()


class NotificationService(NotificationServiceIF):
    """Email notifier"""

    def __init__(self, info: NotificationInfoIF):
        self.info = info

    def send_email(self, template_name, parameters: dict):
        """
        Sends an email using parameters provided

        :param template_name: the email template to use
        :param parameters: any information needed to send the email
        :return:
        """
        smtp_server = "smtp.aoc.nrao.edu"
        sender_email = "do-not-reply@nrao.edu"

        # This gets picked up from whatever's set in the 'user_email' parameter in the capability version parameters
        if isinstance(parameters["destination_email"], list):
            receiver_email = parameters["destination_email"]
        else:
            receiver_email = [parameters["destination_email"]]

        # Add the cc email(s) to the receiver list so the email will be sent to them
        if "cc_email" in parameters and parameters["cc_email"] is not None:
            if isinstance(parameters["cc_email"], list):
                receiver_email.extend(parameters["cc_email"])
            else:
                receiver_email.append(parameters["cc_email"])

        logger.info(f">> In send_email, sending email to: {receiver_email}")
        logger.info(f">> The parameters are: {parameters}")
        template = self.info.lookup_template(template_name)

        if template is None:
            logger.info("No template '" + template_name + "' found.")

        file = template.render(parameters)

        with smtplib.SMTP(smtp_server) as server:
            if not receiver_email:
                return "No PI email found."
            for contact in receiver_email:
                r = server.sendmail(sender_email, contact, file.content.encode("utf8"))

        return r
